"""Functions relating to the handling of bazaar repositories and branches."""

import os

from bzrlib import bzrdir, transport


def ensure_directory_exists(dir_name):
    if os.path.exists(dir_name):
        if not os.path.isdir(dir_name):
            raise RuntimeError("%s exists but isn't a directory", dir_name)
    else:
        os.makedirs(dir_name)


def create_shared_repo(dir_name):
    format = bzrdir.format_registry.make_bzrdir('default')
    to_transport = transport.get_transport(dir_name)
    to_transport.ensure_base()
    newdir = format.initialize_on_transport(to_transport)
    repo = newdir.create_repository(shared=True)
    repo.set_make_working_trees(False)
    return repo


class BazaarRepositoryFactory(object):
    """Responsible for creating repositories for people and projects."""

    def __init__(self, person_root_dir, project_root_dir):
        self.person_root_dir = person_root_dir
        self.project_root_dir = project_root_dir
        ensure_directory_exists(self.person_root_dir)
        ensure_directory_exists(self.project_root_dir)

    def create_shared_repository_for_person(self, person_name):
        """Create an empty shared repo for person."""
        dir_name = os.path.join(self.person_root_dir, person_name)
        create_shared_repo(dir_name)

    def create_shared_repository_for_project(self, project_name):
        """Create a repository for the project with a trunk branch."""
        dir_name = os.path.join(self.project_root_dir, project_name)
        repo = create_shared_repo(dir_name)
        # Now to create the trunk.
        to_transport = repo.user_transport.clone('trunk')
        format = bzrdir.format_registry.make_bzrdir('default')
        branch = bzrdir.BzrDir.create_branch_convenience(
            to_transport.base, format=format, possible_transports=[to_transport])
        branch.set_append_revisions_only(True)
