import logging
import formencode
import formencode.htmlfill

from pylons import config, request
from pylons.controllers.util import redirect
from repoze.what.plugins.pylonshq import ControllerProtector

from webhelpers.paginate import Page

from sloecode.lib.base import BaseController
from sloecode.lib.predicates import has_site_role
from sloecode.model.person import Person, PersonSchema
from sloecode.model.site_role import SiteRole, USER_ADMIN
from sloecode.model.meta import Session
import sloecode.lib.helpers as h

log = logging.getLogger(__name__)


@ControllerProtector(has_site_role(role=USER_ADMIN))
class PersonController(BaseController):
    """PersonController renders pages for the person model administration.

    This includes the basic CRUD functionality.
    """

    def index(self):
        "Render the basic person management page."

        return self.render('/admin/person-manage.html')

    def create(self):
        "Render the basic person creation page."

        return self.render('/admin/person-create.html')

    def process_create(self):
        "Process the person creation request, then redirect to the list view."

        # we use FormEncode to validate the incoming data. First, create the
        # schema for the model object we are trying to validate against:
        schema = PersonSchema()
        try:
            # try and validate it. The formencode.Invalid exception will be
            # thrown if anything is wrong (e.g.- missing fields)
            form_result = schema.to_python(request.params)
        except formencode.Invalid, e:
            # if the exception was thrown, get the error list, and use the
            # htmlfill functionality to re-render the form with error messages
            # added:
            errors = e.unpack_errors()
            return formencode.htmlfill.render(
                self.create(), defaults=request.params, errors=errors)

        # if we get here then we know that everything was OK. We use the
        # 'form_result' dict rather than request.params, since formencode
        # has given us python values, rather than raw strings:
        new_person = Person()
        new_person.login = form_result['login']
        new_person.password = form_result['password']
        new_person.name = form_result['name']
        new_person.email = form_result['email']
        for role in form_result['site_roles']:
            new_person.site_roles.append(SiteRole(role=role))

        Session.add(new_person)
        Session.commit()
        h.flash('Person Added Successfully')
        # Create a shared repo for the person.
        config['bzr.factory'].create_shared_repository_for_person(
            new_person.login)

        return redirect('/a/person/retrieve')

    def retrieve(self):
        "Show a list of users in the system."

        page = Page(Person.get(), page=int(request.params.get('page', 1)))
        return self.render('/admin/person-list.html', {'people': page})

    def update(self, id):
        "Render the person update page of the system."

        # does person exist?
        ppl = Person.get(id=id)
        return self.render('/admin/person-update.html', {'person': ppl[0]})

    def process_update(self, id):
        "Process the person update request then redirect to person list view."
        schema = PersonSchema()
        # we don't care about the login any more:
        del schema.fields['login']
        # we need to set the password & password-verify fields to be optional:
        schema.fields['password'].not_empty = False
        schema.fields['password_verify'].not_empty = False

        try:
            form_result = schema.to_python(request.params)
        except formencode.Invalid, e:
            errors = e.unpack_errors()
            return formencode.htmlfill.render(
                self.update(id), defaults=request.params, errors=errors)

        person_id = id
        person = Person.get(id=person_id)[0]

        # only update password if the user filled it in.
        pw = form_result['password']
        if pw.strip() != "":
            person.password = pw

        person.name = form_result['name']
        person.email = form_result['email']
        # update roles:
        # 1 - remove roles that aren't in form result:
        for role in person.site_roles:
            if role.role not in form_result['site_roles']:
                person.site_roles.remove(role)
            else:
                # if it's in both, remove it from form result since there's
                # nothing to do.
                form_result['site_roles'].remove(role.role)

        # add new roles from form result:
        for new_role in form_result['site_roles']:
            person.site_roles.append(SiteRole(role=new_role))

        Session.merge(person)
        Session.commit()
        h.flash("Person Updated Successfully")
        return redirect('/a/person/retrieve')

    def delete(self, id):
        "Process person delete request then redirect to person list view."

        person = Person.get(id=id)[0]
        # TODO - error checking: Cannot delete the user who is logged in,
        # otherwise people may be able to delete the last admin, leaving the
        # system unusable.
        Session.delete(person)
        Session.commit()
        h.flash("Person Deleted Successfully")
        return redirect('/a/person/retrieve')

