import logging

from pylons import request
from pylons.controllers.util import redirect, abort
from repoze.what.plugins.pylonshq import ControllerProtector

from sloecode.lib.base import BaseController
from sloecode.model.project import Project
from sloecode.model.person import Person
from sloecode.model.membership import Membership
from sloecode.model.site_role import PROJECT_ADMIN
from sloecode.model.project_role import ProjectRole
from sloecode.model.meta import Session
from sloecode.lib.predicates import has_site_role
import sloecode.lib.helpers as h

log = logging.getLogger(__name__)

@ControllerProtector(has_site_role(PROJECT_ADMIN))
class ProjectController(BaseController):
    """ProjectController contains basic admin functionality for the Project
    model.

    This includes basic CRUD information.
    """

    def index(self, project_name):
        """Render project details page.
        """
        projects = Project.get(name=project_name)
        existing_project = projects[0] if projects else None

        if existing_project is None:
            abort(404)
        else:
            return self.render('/project-details.html',
                               {'project': existing_project})

    def manage_users(self, project_name):
        """
        Add/Remove users from this project.
        """
        all_people = Person.get()
        # Need to check here, perhaps the project has been deleted between then
        # & now
        this_project = Project.get(name = project_name)[0]

        if this_project is None:
            abort(404)

        # Figure out whos already in this project and their role in it
        already_assigned = {}
        for user in this_project.users:
            already_assigned[user.person.login] = user.roles[0].role

        for user in all_people:
            user.sc_already_assigned = already_assigned.get(user.login, False)

        return self.render('/project-manage-users.html',
                           {'all_people': all_people,
                            'project_name': project_name})

    def process_manage_users(self, project_name):
        """
        Process the change in the users-in-projects
        """

        project_details = Project.get(name = project_name)[0]
        selected_people = request.params.getall('usernames')

        # Update or remove existing users from the project
        for memship in project_details.users:
            users_role = request.params.get('user_roles_' \
                                            + memship.person.login)
            if memship.person.login in selected_people:
                if memship.roles[0].role != users_role:
                    memship.roles[0].role = users_role
                    h.flash("Role for %s Updated to %s Successfully"
                            % (memship.person.login, users_role))
                selected_people.remove(memship.person.login)
            elif memship.person.login not in selected_people:
                Session.delete(memship)
                h.flash("%s Removed from Project Successfully"
                        % memship.person.login)

        Session.commit()

        # Add new users to the project
        for user in selected_people:
            user_details = Person.get(login = user)[0]
            new_membership = Membership(person = user_details,
                                        project = project_details)

            Session.add(new_membership)

            # And setup the project roles
            user_project_role = ProjectRole(membership = new_membership)
            user_project_role.role = request.params.get('user_roles_' + user)

            Session.add(user_project_role)
            Session.commit()

            h.flash("%s Successfully Added to Project"
                    % user_details.name)

        return redirect('/p/%s' % project_name)
