"""Code for setting up web-app authentication & authorisation.
"""

from repoze.what.middleware import setup_auth
from repoze.who.plugins.sa import SQLAlchemyAuthenticatorPlugin
from repoze.who.plugins.sa import SQLAlchemyUserMDPlugin
from repoze.who.plugins.form import RedirectingFormPlugin
from repoze.who.plugins.auth_tkt import AuthTktCookiePlugin

from sloecode.model.meta import Session
from sloecode.model import Person


def add_auth(app):
    """
    Add authentication and authorization middleware to the ``app``.

    :param app: The WSGI application.
    :return: The same WSGI application, with authentication and
    authorization middleware.

    People will login using HTTP Authentication and their credentials are
    kept in an ``Htpasswd`` file. For authorization through repoze.what,
    we load our groups stored in an ``Htgroups`` file and our permissions
    stored in an ``.ini`` file.

    """

    # we use the SQLAlchemy repoze.who plugin for authentication:
    authenticator = SQLAlchemyAuthenticatorPlugin(Person, Session)
    # override the default names for the login column, and the verification 
    # method:
    authenticator.translations['user_name'] = 'login'
    authenticator.translations['validate_password'] = 'verify_password'

    # We also want to use the SA metadata providor:
    mdprovidor = SQLAlchemyUserMDPlugin(Person, Session)
    mdprovidor.translations['user_name'] = 'login'

    # we use a redirecting form identifier:
    id_form = RedirectingFormPlugin('/auth/login', 
                                    '/auth/dologin', 
                                    '/auth/logout', 
                                    'auth_tkt')
    # TODO - we can make this much more secure if we want.
    auth_tkt = AuthTktCookiePlugin('S3cr3tStr1ng')

    identifiers = [('form', id_form),('auth_tkt', auth_tkt)]
    authenticators = [('sa_auth', authenticator)]
    challengers = [('form', id_form)]
    mdproviders = [('sa_md', mdprovidor)]


    app_with_auth = setup_auth(
        app,
        identifiers=identifiers,
        authenticators=authenticators,
        challengers=challengers,
        mdproviders=mdproviders)
    return app_with_auth

