"""Person model"""
import hashlib
import formencode
import formencode.variabledecode

from sqlalchemy import Column
from sqlalchemy.types import Integer, String
from sqlalchemy.orm import relationship, synonym

from sloecode.model.meta import Base, QueryMixin

class Person(Base, QueryMixin):
    """A model for a person in the sloecode system.

    People have usernames and passwords for authentication to the system, as
    well as other metadata such as real (display) names and email addresses.

    Users also have a flag associated with them that indicated whether they
    are a site-admin, or a regular user.

    Finally, this model has links to the AuthKey model (which stores SSH
    authentication keys), and the Project model (via the Membership model).
    """
    __tablename__ = "person"

    id = Column(Integer, primary_key=True)
    login = Column(String(200), unique=True)
    # Store password hash in DB. Passwords are hashed transparently using the
    # synonym created below.
    _password = Column('password', String(200))

    name = Column(String(200))
    email = Column(String(200))

    # Stuff stored in other tables:

    # Note that we only need to do this once - the backref value will be created
    # in the authkey object:
    auth_keys = relationship("AuthKey", order_by="AuthKey.id", backref="user")

    #TODO rename this to 'memberships' or similar. Will need massive renaming
    # effort throughout the project (especially unit tests).
    projects = relationship("Membership", backref="person")
    site_roles = relationship('SiteRole', backref="person")

    def verify_password(self, password):
        """Verify that the password given to us is the correct one.
        """
        return self.password == self._hashme(password)

    def _get_password(self):
        """return the hashed password.
        """
        return self._password

    def _set_password(self, password):
        """hash the password on entry.
        """
        self._password = self._hashme(password)

    @staticmethod
    def _hashme(content):
        """Return a hashed version of content.
        """
        hash_algorithm = hashlib.sha256(content)
        # TODO - add salt!
        return hash_algorithm.hexdigest()

    password = synonym('_password', 
                       descriptor=property(_get_password, _set_password))


class UniqueLogin(formencode.FancyValidator):
    """A simple formencode validator that checks that Person logins are unique.
    """

    def _to_python(self, value, state):
        """Try to validate form data.
        """
        if len(Person.get(login=value.strip)) > 0:
            raise formencode.Invalid(
                'That login already exists.', value, state)
        return value


class PersonSchema(formencode.Schema):
    "A formencode schema for the Person object."

    allow_extra_fields = True
    filter_extra_fields = True

    # pre-validators are run before the actual data validators below. This
    # particular validator allows us to convert the flat HTML kay/value dict
    # to a nested python structure. We need this since the form has multiple
    # checkboxes that represent the site roles.
    pre_validators = [formencode.variabledecode.NestedVariables()]

    # logins must be unique and plain text. Should we enforce a minimum
    # length?
    login = formencode.All(UniqueLogin(not_empty=True, strip=True),
        formencode.validators.PlainText(not_empty=True))
    # should we enforce a minimum password security level? There's a
    # pre-built FE validator for that...
    password = formencode.validators.String(not_empty=True)
    password_verify = formencode.validators.String(not_empty=True)
    name = formencode.validators.String(not_empty=True)
    email = formencode.validators.Email(not_empty=True)
    # grab multiple site role names:
    site_roles = formencode.ForEach(formencode.validators.String())

    chained_validators = [formencode.validators.FieldsMatch(
                            'password', 'password_verify')]
