#!/usr/bin/env python

from sqlalchemy.exc import IntegrityError
from unittest import TestCase

from sloecode.model.meta import Session, Base, engine
from sloecode.model.person import Person

class TestPerson(TestCase):
    def setUp(self):
        # create all DB tables:
        Base.metadata.create_all(engine)

    def tearDown(self):
        # remove all DB tables.
        Base.metadata.drop_all(engine)
        # remove session - avoids a memory leak, apparently.
        Session.remove()

    def test_create(self):
        "Create a person."
        p1 = Person(login='thomir', name='Thomi Richards', email='thomir@somewhere.com')
        self.failIf(p1 is None)

    def test_unique(self):
        "Make sure the login column really is unique."
        p1 = Person(login='thomir', name='Thomi the first')
        Session.add(p1)
        Session.commit()

        p2 = Person(login='thomir', name='Thomi the second')
        Session.add(p2)
        self.failUnlessRaises(IntegrityError, Session.commit)

    def test_tests(self):
        "make sure DB data is flushed between tests. This test will fail if it's not."
        p1 = Person(login='thomir')
        Session.add(p1)
        Session.commit()

    def test_querying(self):
        "make sure we can query for specific users once they're in the DB."
        # Add some users:
        Session.add(Person(login='thomir', name='Thomi Richards', email='thomir@gmail.com'))
        Session.add(Person(login='joeb', name='Joe Bloggs', email='joeb@somewhere.com'))
        Session.add(Person(login='timmy', email='timmy@lod.com'))
        Session.commit()

        # query all people:
        qi = Session.query(Person)
        self.assertTrue(len(qi.all()) == 3)

        # query all people with specific login:
        qi = Session.query(Person).filter_by(login='timmy')
        self.assertTrue(len(qi.all()) == 1)

        # query all people with specific name:
        qi = Session.query(Person).filter(Person.name.like('Joe%'))
        self.assertTrue(len(qi.all()) == 1)

        # query where name is empty:
        qi = Session.query(Person).filter(Person.name == None)
        self.assertTrue(len(qi.all()) == 1)

    def test_new_style_query(self):
        "Test the QueryMixin query helper methods."

        Session.add(Person(login='thomir', name='Thomi Richards', email='thomir@gmail.com'))
        Session.add(Person(login='joeb', name='Joe Bloggs', email='joeb@somewhere.com'))
        Session.add(Person(login='timmy', email='timmy@lod.com'))
        Session.commit()

        ppl = Person.get(login='thomir')
        self.assertTrue(len(ppl) == 1)

    def test_update(self):
        "Make sure we can edit a user in the database."

        Session.add(Person(login='thomi', name='thomas'))
        Session.commit()

        me = Person.get(login='thomi')[0]
        me.name = 'Thomi Richards'
        me.login='thomir'
        Session.merge(me)
        Session.commit()

        thomi = Person.get(login='thomir')[0]
        self.assertTrue(thomi.name == 'Thomi Richards')
        self.assertTrue(thomi is me)

    def test_password_hash(self):
        'Test that we hash passwords correctly'

        me = Person(login='foo', password='secret')

        self.assertTrue(me.password != 'secret')

    def test_password_validation(self):
        'Test that we can validate passwords correctly.'
        me = Person(password='secret')

        import hashlib
        h = hashlib.new('sha256', 'secret')
        self.assertTrue(me.password == h.hexdigest())



