
from unittest import TestCase
from pylons import request

from sloecode.model.meta import Base, Session, engine
from sloecode.model import Person, Project, Membership, ProjectRole, SiteRole
from sloecode.lib.security import has_site_role, has_project_role



class TestSecurity(TestCase):
    """Test security functions.
    """

    def setUp(self):
        # create all DB tables:
        Base.metadata.create_all(engine)

    def tearDown(self):
        # remove all DB tables.
        Base.metadata.drop_all(engine)
        # remove session - avoids a memory leak, apparently.
        Session.remove()

    def test_has_site_role(self):
        """Test the has_site_role method works as expected.
        """
        user_a = Person(login='user_a', site_roles =
            [SiteRole(role='User Admin'), SiteRole(role='Project Admin')])
        user_b = Person(login='user_b', site_roles =
            [SiteRole(role='User Admin')])
        user_c = Person(login='user_c', site_roles =
            [SiteRole(role='Project Admin')])

        Session.add(user_a)
        Session.add(user_b)
        Session.add(user_c)
        Session.commit()

        self.assertTrue(has_site_role('Project Admin', user_a))
        self.assertTrue(has_site_role('User Admin', user_a))
        self.assertTrue(has_site_role('made up', user_a) == False)
        self.assertTrue(has_site_role('Project Admin', user_b) == False)
        self.assertTrue(has_site_role('User Admin', user_b))
        self.assertTrue(has_site_role('Project Admin', user_c))
        self.assertTrue(has_site_role('User Admin', user_c) == False)

    def test_has_project_role(self):
        """Test the has_project_role function works as expected.
        """
        # create a project:
        sc = Project(name='sloecode')
        Session.add(sc)

        # one user who is not on the project at all:
        user_a = Person(login='user_a')
        Session.add(user_a)

        # one user who is an observer on sc:
        user_b = Person(login='user-b',
            projects=[Membership(project=sc,
                roles=[ProjectRole(role='Observer')])])
        Session.add(user_b)

        # one user who is a developer on sc:
        user_c = Person(login='user-c',
            projects=[Membership(project=sc,
                roles=[ProjectRole(role='Developer')])])
        Session.add(user_c)

        # one user who is a manager on sc:
        user_d = Person(login='user-d',
            projects=[Membership(project=sc,
                roles=[ProjectRole(role='Manager')])])
        Session.add(user_d)

        Session.commit()

        # first - ensure that user_a has no roles:
        # we specify the project in three separate ways:
        self.assertTrue(has_project_role(sc, 'Observer', user_a) == False)
        self.assertTrue(has_project_role('sloecode', 'Developer', user_a)
            == False)
        self.assertTrue(has_project_role(sc.id, 'Manager', user_a) == False)

        # now repeat those tests, but with users with more rights:
        self.assertTrue(has_project_role(sc, 'Observer', user_b) == True)
        self.assertTrue(has_project_role('sloecode', 'Developer', user_b)
            == False)
        self.assertTrue(has_project_role(sc.id, 'Manager', user_b) == False)

        self.assertTrue(has_project_role(sc, 'Observer', user_c) == False)
        self.assertTrue(has_project_role('sloecode', 'Developer', user_c)
            == True)
        self.assertTrue(has_project_role(sc.id, 'Manager', user_c) == False)

        self.assertTrue(has_project_role(sc, 'Observer', user_d) == False)
        self.assertTrue(has_project_role('sloecode', 'Developer', user_d)
            == False)
        self.assertTrue(has_project_role(sc.id, 'Manager', user_d) == True)
